from typing import List, Optional

from sqlalchemy.orm import Session

from app.models.user import User
from app.schemas.user import UserCreate, UserUpdate


def get_user(db: Session, id: int) -> Optional[User]:
    """
    Get a user by ID.
    """
    return db.query(User).filter(User.id == id).first()


def get_user_by_email(db: Session, email: str) -> Optional[User]:
    """
    Get a user by email.
    """
    return db.query(User).filter(User.email == email).first()


def get_users(db: Session, skip: int = 0, limit: int = 100) -> List[User]:
    """
    Get multiple users.
    """
    return db.query(User).offset(skip).limit(limit).all()


def create_user(db: Session, obj_in: UserCreate) -> User:
    """
    Create a new user.
    """
    # In a real application, you'd hash the password
    db_obj = User(
        email=obj_in.email,
        username=obj_in.username,
        hashed_password=obj_in.password,  # This should be hashed in a real app
        is_active=obj_in.is_active,
        is_superuser=obj_in.is_superuser,
    )
    db.add(db_obj)
    db.commit()
    db.refresh(db_obj)
    return db_obj


def update_user(db: Session, db_obj: User, obj_in: UserUpdate) -> User:
    """
    Update a user.
    """
    update_data = obj_in.dict(exclude_unset=True)

    # Handle password updates
    if "password" in update_data and update_data["password"]:
        # In a real application, you'd hash the password
        update_data["hashed_password"] = update_data.pop("password")

    for field, value in update_data.items():
        setattr(db_obj, field, value)

    db.add(db_obj)
    db.commit()
    db.refresh(db_obj)
    return db_obj


def delete_user(db: Session, id: int) -> None:
    """
    Delete a user.
    """
    user = db.query(User).get(id)
    if user:
        db.delete(user)
        db.commit()
